/* Main code for PluckGet */

#include <PalmOS.h>

#include "pppsetRsc.h"

Err errno=0;
Err h_errno=0;
UInt16 AppNetRefnum;
UInt32 AppNetTimeout;
NetHostInfoBufType AppHostInfo;

char **iface = NULL;

Boolean close_netlib = true;

static void 
close_library(void)
{
    if (close_netlib)
	NetLibClose(AppNetRefnum,false);
}


/* open the library */
static int 
open_library(void)
{
	Err error;
	Err ifErrs;
	
	error = SysLibFind("Net.lib",&AppNetRefnum);
	ErrFatalDisplayIf(error,"Can't find Net.lib");
	
	AppNetTimeout = SysTicksPerSecond()*60;
	
	error = NetLibOpen(AppNetRefnum,&ifErrs);
	if (error == netErrAlreadyOpen) {
	    /* The open count was incremented, we will close as not to 
	       add additional increment
	    */
	    close_library();
	    close_netlib = false;
	} else if (error) 
	    ErrFatalDisplayIf(error,"Can't open Net.lib");
	
	return 0;
}

void *
GetObjectPtr(UInt16 objectID)
{
        FormPtr frm;

        frm=FrmGetActiveForm();
        return (FrmGetObjectPtr(frm,FrmGetObjectIndex(frm,objectID)));
}

static void
update_checkbox(UInt16 i)
{
    UInt16 ifinstance;
    UInt32 ifcreator;
    Err status;
    UInt8 flag=0;
    UInt16 valuelen;
    
    status = NetLibIFGet(AppNetRefnum,i,&ifcreator,&ifinstance);
    ErrFatalDisplayIf(status,"update_checkbox1");
    
    valuelen = sizeof(flag);
    status = NetLibIFSettingGet(AppNetRefnum,ifcreator,ifinstance,
				    netIFSettingVJCompEnable,&flag,&valuelen);
    ErrFatalDisplayIf(status,"update_checkbox2");
    CtlSetValue(GetObjectPtr(chkComp),flag);
}

static void
update_vjcomp(UInt16 i)
{
    UInt16 ifinstance;
    UInt32 ifcreator;
    Err status;
    UInt8 flag;
    UInt16 valuelen;
    
    status = NetLibIFGet(AppNetRefnum,i,&ifcreator,&ifinstance);
    ErrFatalDisplayIf(status,"update_vjcomp");
	
    valuelen = sizeof(flag);
    flag = CtlGetValue(GetObjectPtr(chkComp));
    status = NetLibIFSettingSet(AppNetRefnum,ifcreator,ifinstance,
				netIFSettingVJCompEnable,&flag,valuelen);
    ErrFatalDisplayIf(status,"update_vjcomp2");
}

static void
iface_up(UInt16 i)
{
    UInt16 ifinstance;
    UInt32 ifcreator;
    Err status;
    
    status = NetLibIFGet(AppNetRefnum,i,&ifcreator,&ifinstance);
    ErrFatalDisplayIf(status,"netlibifget1");
    NetLibIFUp(AppNetRefnum,ifcreator,ifinstance);
}

static void
iface_down(UInt16 i)
{
    UInt16 ifinstance;
    UInt32 ifcreator;
    Err status;
    
    status = NetLibIFGet(AppNetRefnum,i,&ifcreator,&ifinstance);
    ErrFatalDisplayIf(status,"netlibifget2");
    NetLibIFDown(AppNetRefnum,ifcreator,ifinstance,200);
}

static void
MainFormInit(void)
{
	ListPtr list = GetObjectPtr(lstIfaces);
	UInt16 i;
	UInt16 ifacecount;
	UInt16 ifinstance;
	UInt32 ifcreator;
	Err status;
	char name[32];
	UInt16 valuelen;

	/* Count the interfaces */
	for (i=0,ifacecount=0;;i++) {
	    status = NetLibIFGet(AppNetRefnum,i,&ifcreator,&ifinstance);
	    if (status == 0) {
		ifacecount++;
	    } else 
		break;
	}
	/* Allocate structure */
	iface = MemPtrNew(sizeof(*iface)*ifacecount);
	/* Fill structure */

	for (i=0;i<ifacecount;i++) {
	    status = NetLibIFGet(AppNetRefnum,i,&ifcreator,&ifinstance);
	    if (status == 0) {
		valuelen = 32;
		status = NetLibIFSettingGet(AppNetRefnum,ifcreator,ifinstance,
					    netIFSettingName,name,&valuelen);
		ErrFatalDisplayIf(status,"GetName");
		iface[i] = MemPtrNew(valuelen+1);
		StrCopy(iface[i],name);
	    } else {
		ifacecount = i;
		break;
	    }
	}
	LstSetListChoices(list,iface,ifacecount);
	if (ifacecount)
	    update_checkbox(0);
	CtlSetValue(GetObjectPtr(chkRemember),!close_netlib);
}

static Boolean MainFormHandleEvent (EventPtr event)
{
	Boolean handled = false;	
	int i;
	
	switch (event->eType) {
	case ctlSelectEvent:
	    switch(event->data.ctlSelect.controlID) {
	    case chkComp:
		i = LstGetSelection(GetObjectPtr(lstIfaces));
		if (i == noListSelection)
		    break;
		update_vjcomp(i);
		handled = true;
		break;
	    case chkRemember:
		close_netlib = !CtlGetValue(GetObjectPtr(chkRemember));
		handled = true;
		break;
	    case btnUp:
	    case btnDown:
		i = LstGetSelection(GetObjectPtr(lstIfaces));
		if (i == noListSelection)
		    break;
		if (event->data.ctlSelect.controlID == btnUp)
		    iface_up(i);
		else
		    iface_down(i);
		handled = true;
		break;
	    case btnAbout:
		FrmAlert(frmAbout);
		handled = true;
		break;
	    case btnHelp:
		FrmHelp(strHelp);
		handled = true;
		break;
	    default:
		handled=false;
	    }
	    
	    break;
	case frmOpenEvent:
	    MainFormInit();
	    FrmDrawForm(FrmGetActiveForm());
	    handled = true;
	    break;
		    
	 case frmCloseEvent:
	     /* Important, this should be handled somewhere else */
	     handled = false;
	     break;		
	 case lstSelectEvent:
	     update_checkbox(event->data.lstSelect.selection);
	     handled = true;
	     break;
	 case menuEvent:
		MenuEraseStatus(NULL);
		
		switch(event->data.menu.itemID) {
		}
		
		handled = true;
		break;
		
	 default:
	    break;
	}
	
	return handled;
}

static Boolean ApplicationHandleEvent(EventPtr e)
{
	FormPtr frm;
	Int16    formId;
	Boolean handled = false;
	
	if (e->eType == frmLoadEvent) {
		formId = e->data.frmLoad.formID;
		frm = FrmInitForm(formId);
		FrmSetActiveForm(frm);
		
		switch(formId) {
		 case MainForm:
			FrmSetEventHandler(frm, MainFormHandleEvent);
			break;
		}
		handled = true;
	}
	
	return handled;
}

/* Get preferences, open (or create) app database */
static int 
StartApplication(void)
{
    open_library();
    FrmGotoForm(MainForm);
    return 0;
}

/* Save preferences, close forms, close app database */
static void 
StopApplication(void)
{
    FrmSaveAllForms();
    FrmCloseAllForms();
    close_library();
}

/* The main event loop */
static void EventLoop(void)
{
    Err err;
    EventType e;

    do {
	EvtGetEvent(&e, evtWaitForever);
	if (! SysHandleEvent (&e))
	    if (! MenuHandleEvent (NULL, &e, &err))
		if (! ApplicationHandleEvent (&e))
		    FrmDispatchEvent (&e);
    } while (e.eType != appStopEvent);
}

/* Main entry point; it is unlikely you will need to change this except to
   handle other launch command codes */
UInt32
PilotMain(UInt16 launchCode, MemPtr cmdPBP, UInt16 launchFlags)   
{
    Err err;
    
    if (launchCode == sysAppLaunchCmdNormalLaunch) {
	
	err = StartApplication();
	if (err) return err;
	
	EventLoop();
	StopApplication();
    } else {
	return sysErrParamErr;
    }
    
    return 0;
}
